package fr.doap.llog;

import java.util.Set;

public class LogLooper extends ThreadLooper {
	
	public void post(LogMsg aLogMsg) {
		add(new LLMsg(aLogMsg));
	}
	
	public void put(String aKey, Filter aF) {
		mFilters.put(aKey, aF);
	}
	
	public class LLMsg implements WorkItem {
		public LLMsg(LogMsg aLogMsg) { mLogMsg = aLogMsg; }
		
		@Override
		public void run() {
			int lI = mLogMsg.getCallerIndex();
			StackTraceElement lS = mLogMsg.mSTE[lI];
			String lClass = lS.getClassName();
			lClass = lClass.substring(lClass.lastIndexOf('.'));
			if (mLogMsg.mTag == null || mLogMsg.mTag.isEmpty()) mLogMsg.mTag = lClass;

			String lC = lS.getClassName() + "." + lS.getMethodName();
			Filter lF = mFilters.get(lC);
			if (lF == null) lF = Filter.DEFAULT;
			boolean lShowST = false;
			if (mLogMsg.mSeverity >= lF.mLogLevel) {
				mLogger.logHead(mLogMsg, lS);
				if (mLogMsg.mSeverity >= lF.mSTLevel) {
					lShowST = true;
					for (int i=lI+1; i<mLogMsg.mSTE.length; i++) mLogger.logSTLine(mLogMsg, mLogMsg.mSTE[i]);
				}
			}
			if (mLogMsg.mThrowable != null) printLog(mLogMsg.mThrowable, lF, lShowST);
		}
		
		protected void printLog(Throwable aThrowable, Filter aFilter, boolean aShowST) {
			StackTraceElement[] lSTEs = aThrowable.getStackTrace();
			StackTraceElement lSTE = lSTEs[0];
			String lClass = lSTE.getClassName();
			lClass = lClass.substring(lClass.lastIndexOf('.'));
			String lC = lSTE.getClassName() + "." + lSTE.getMethodName();
			String lMsg = aThrowable.getLocalizedMessage();
			if (aFilter != null) {
				mLogger.logCauseHead(mLogMsg, aThrowable);
				if (aShowST) for (int i=1; i<lSTEs.length; i++) mLogger.logSTLine(mLogMsg, lSTEs[i]);
			} else {
				System.out.println("caused by " + lC + ": " + lMsg + " (" + lSTE.getFileName() + ":" + lSTE.getLineNumber() + ")");
			}
			if (aThrowable.getCause() != null) printLog(aThrowable.getCause(), aFilter, aShowST);
		}
		
		private LogMsg	mLogMsg;
	}
	
	private	Dict<Filter>	mFilters = new Dict<>();
	private ILogger			mLogger = ILogger.CONSOLE;
}
